@@ -11,6 +11,7 @@ from .log import log
 
 load_dotenv()
 load_dotenv("{d}/.env".format(d=os.getcwd()))
+load_dotenv("{d}/../.env".format(d=os.getcwd()))
 
 def _getenv(variable:str , default:str):
         '''
@@ -102,13 +103,14 @@ class KnowledgeGraph:
 
         try:
             records, summary, keys = self.driver.execute_query(cql, database_=self.database,)
-            log.info("The query `{query}` returned {records_count} records in {time} ms.".format(
+            log.debug("The query `{query}` returned {records_count} records in {time} ms.".format(
                 query=summary.query, records_count=len(records),
                 time=summary.result_available_after))
             return records, summary
         except Exception as e:
             log.error(e)
 
+
     def neomodel_query(self,cypher_query, params: dict={}):
         '''
         Performs a cypher query in the standard neomodel method, which will return a list of lists of neomodel.
@@ -116,17 +118,23 @@ class KnowledgeGraph:
         '''
         if self.driver == None:
             self.connect()
+
+        print(nm.db.driver)
         
         return nm.db.cypher_query(cypher_query, resolve_objects=True, params=params)
 
-    def connect(self, driver=None):
+
+    def connect(self, driver = None):
         '''
         Ensures that a connection to Neo4j has been established and is functioning.
         '''
         try:
-            if driver==None:
-                driver = GraphDatabase.driver(self.connection, auth=(self.user,self.password))
-            
+            if driver is None:
+                if self.driver is None:
+                    driver = GraphDatabase.driver(self.connection, auth=(self.user,self.password))
+                else:
+                    driver = self.driver
+    
             log.info("Testing Neo4j Connectivity...")
             driver.verify_connectivity()
             
@@ -145,19 +153,24 @@ class KnowledgeGraph:
                 log.error("Unable to log in to Neo4J for KnowledgeGraph. Check that your user and password are correct.")
                 return False
             
-            self.driver=driver
-            return True
+            self.driver = driver
+            nm.config.DRIVER = self.driver
+            nm.db.set_connection(driver=self.driver)
+
+            return
+        
         except Exception as e:
             log.error(e)
             log.error("There was an issue connecting to Neo4J server, please verify that it is running and configured correctly.")            
             raise e
     
+
     def getNeo4jDriver(self):
         '''
         Returns the Neo4j driver class being used (usually neo4j.GraphDatabase.driver) so that direct neo4j calls can be
         made or it can be passed into other libraries.
         '''
-        if self.driver==None:
+        if self.driver is None:
             self.connect()
         return self.driver
 
@@ -166,7 +179,7 @@ class KnowledgeGraph:
         Shuts down the current connection to neo4j. This should be done at the end of each program / notebook.
         If you use the "with statement" context control, this will be called automatically when the code block ends.
         '''
-        if self.driver!=None:
+        if self.driver is not None:
             self.driver.close()
         self.driver=None
 
@@ -176,7 +189,7 @@ class KnowledgeGraph:
         So, we can use this to determine if an object has been saved or not and respond appropriately.
         '''
         if node.uid is None:
-            log.warn("Nodes must be persisted with node.save() before being connected. Forcing this now.")
+            log.warning("Nodes must be persisted with node.save() before being connected. Forcing this now.")
             return node.save()
         return node
 
@@ -204,7 +217,8 @@ class KnowledgeGraph:
         So, cypher_query("MATCH (a)-[b]->(c) RETURN a","a") would give the same result list by dropping the "b" and "c" results.
         Reduce the risk of Cypher injection attacks by passing Cypher parameters in params.
         '''
-        if self.driver == None:
+        if self.driver is None:
+            log.debug("Cypher Query made before driver created, creating connection driver.")
             self.connect()
         
         results, meta = nm.db.cypher_query(cypher_query, resolve_objects=True, params=params)
@@ -214,7 +228,7 @@ class KnowledgeGraph:
             index=meta.index(extract_variable)
         else:
             if len(meta)>1:
-                log.warn("cypher_query is defaulting to return the first element of multiple return values in results.")      
+                log.warning("cypher_query is defaulting to return the first element of multiple return values in results.")      
 
         return [item[index] for item in results]
 
@@ -393,7 +407,7 @@ class KnowledgeGraph:
         if source is None:
             source= Source(name=name).save()
         else:
-            log.warn("Found existing Source node for name {name}, returning pre-existing node.".format(name=name))
+            log.warning("Found existing Source node for name {name}, returning pre-existing node.".format(name=name))
         return source
 
     def get_source_by_uid(self, uid: str) -> Source:
@@ -448,7 +462,7 @@ class KnowledgeGraph:
             content=Content(url=url,date=date,title=title,text=text, type=type).save()
             content.is_from.connect(source)
         else:
-            log.warn("Found existing Content node for url {url}, returning pre-existing node.".format(url=url))
+            log.warning("Found existing Content node for url {url}, returning pre-existing node.".format(url=url))
         return content
 
     def get_content_by_uid(self, uid:str) -> Content:
@@ -509,9 +523,9 @@ class KnowledgeGraph:
     # Active Data / Entity
     # -----------------
 
-    def add_entity(self, content:Content, text:str, type:str ) -> Entity:
+    def add_entity(self, node:Content|Request, text:str, type:str ) -> Entity:
         entity=Entity(text=text,type=type).save()
-        entity.mentioned_in.connect(content)
+        entity.mentioned_in.connect(node)
         return entity
     
     def add_refers_to_edge(self, entity:Entity, reference:ReferenceNode, confidence=1.0 ):
@@ -581,7 +595,7 @@ class KnowledgeGraph:
         if user is None:
             user=User(name=name, email=email).save()
         else:
-            log.warn("Found existing User node for name {name}, returning pre-existing node.".
+            log.warning("Found existing User node for name {name}, returning pre-existing node.".
                       format(name=name))
         return user      
 
@@ -623,7 +637,7 @@ class KnowledgeGraph:
             request = Request(title=title, text=text).save()
             request.requested_by.connect(user)
         else:
-            log.warn("Found existing Request node for {title}, returning pre-existing node.".format(title=title))
+            log.warning("Found existing Request node for {title}, returning pre-existing node.".format(title=title))
         return request
 
     def get_request_by_uid(self, uid:str) -> Request:
@@ -699,7 +713,7 @@ class KnowledgeGraph:
             recommendation.recommends.connect(content)
             recommendation.relates_to.connect(request)
         else:
-            log.warn("Found existing Recommendation node between content and request, returning pre-existing node.")
+            log.warning("Found existing Recommendation node between content and request, returning pre-existing node.")
         return recommendation
     
     def get_recommendation(self, content:Content, request:Request) -> Recommendation:
@@ -748,21 +762,31 @@ class KnowledgeGraph:
             if recommendation is not None:
                 summary.focus_on.connect(recommendation)
         else:
-            log.warn("Found existing Summary node, returning pre-existing node.")
+            log.warning("Found existing Summary node, returning pre-existing node.")
         return summary
     
-    def get_summary(self, content:Content, recommendation:Recommendation) -> Summary:
-        return self.cypher_query_one("""
-            MATCH (r:Recommendation) WHERE r.uid=$recommendation_id
-            MATCH (a:Content) WHERE a.uid=$content_id
-            MATCH (s:Summary)
-            MATCH (s)-[x:SUMMARIZES]->(a)
-            MATCH (s)-[y:FOCUS_ON]->(r)
-            RETURN s """,
-            "s", params={
-                "recommendation_id": recommendation.uid,
-                "content_id": content.uid})
-    
+    def get_summary(self, content:Content, recommendation:Recommendation=None) -> Summary:
+        # Summaries without recommendations should explictly NOT have recommendations associated with them
+        if recommendation is None:
+            return self.cypher_query_one("""
+                MATCH (a:Content) WHERE a.uid=$content_id
+                MATCH (s:Summary)
+                MATCH (s)-[x:SUMMARIZES]->(a)
+                WHERE NOT (s)-[:FOCUS_ON]->()
+                RETURN s """,
+                "s", params={"content_id": content.uid})
+        else:
+            return self.cypher_query_one("""
+                MATCH (r:Recommendation) WHERE r.uid=$recommendation_id
+                MATCH (a:Content) WHERE a.uid=$content_id
+                MATCH (s:Summary)
+                MATCH (s)-[x:SUMMARIZES]->(a)
+                MATCH (s)-[y:FOCUS_ON]->(r)
+                RETURN s """,
+                "s", params={
+                    "recommendation_id": recommendation.uid,
+                    "content_id": content.uid})
+        
     def get_summary_by_id(self, uid:str) -> Summary:
         summary = Summary.nodes.get_or_none(uid=uid)
         return summary
@@ -803,7 +827,7 @@ class KnowledgeGraph:
             tldr = Tldr(date=date).save()
             tldr.response_to.connect(request)
         else:
-            log.warn("Found existing Tldr node for request on given date, returning pre-existing node.")
+            log.warning("Found existing Tldr node for request on given date, returning pre-existing node.")
         return tldr
 
     def get_tldr(self, request: Request, date: datetime.date) -> Tldr:
@@ -911,7 +935,7 @@ class KnowledgeGraph:
         else:
             feedback.click_date=date
             feedback.save()
-            log.warn("Updated click date of existing Feedback.")
+            log.warning("Updated click date of existing Feedback.")
 
         return feedback
 
@@ -925,7 +949,7 @@ class KnowledgeGraph:
         else:
             feedback.score=score
             feedback.save()
-            log.warn("Updated scoring of existing Feedback.")
+            log.warning("Updated scoring of existing Feedback.")
 
         return feedback
 
